#ifndef _KNIGHTMARE_APP
#define _KNIGHTMARE_APP

#include "SDL_ttf.h"

#include "BatAnimation.h"
#include "Lightning.h"

// This is to have a 640x480 screen:
#define PERSPECTIVE_DISTANCE -895.8F

#define MAX_HIGHSCORES 10

#define STATE_NONE -1
#define STATE_SPLASH  0
#define STATE_MSX  2
#define STATE_KONAMI  3
#define STATE_TITLE  4
#define STATE_GAMESTART 5
#define STATE_GAME  6
#define STATE_GAMEOVER 7
#define STATE_ENDSEQUENCE 8
#define STATE_TITLEANIMATION 9
#define STATE_INTERLEVEL  10
#define STATE_HOWTOPLAY  11
#define STATE_CREDITS  12

#define STATE_TEST  13

#define KKEY_UP 0
#define KKEY_RIGHT 1
#define KKEY_DOWN 2
#define KKEY_LEFT 3
#define KKEY_FIRE 4
#define KKEY_PAUSE 5
#define KKEY_QUIT 6

#define PAUSESTATE_PLAYING  0
#define PAUSESTATE_PAUSING  1
#define PAUSESTATE_PAUSED  2
#define PAUSESTATE_UNPAUSING 3

#define MENU_MAIN		0
#define MENU_OPTIONS	1
#define MENU_INPUT1		2
#define MENU_INPUT2		3


class MenuItem {
public:
	MenuItem(char *text,int ID) {
		m_text = new Symbol(text);
		m_ID = ID;
	}
	~MenuItem() {
		delete m_text;
	}

	Symbol *m_text;
	int m_ID;
};

class KnightmareApp
{
    public:
        KnightmareApp();
        ~KnightmareApp();

        bool cycle(KEYBOARDSTATE *k);
        void draw();

        void save_configuration(void);
        void load_configuration(void);

		void MouseClick(int mx,int my);

        void save_hiscores(void);
        void load_hiscores(void);

    protected:
        int splash_cycle(KEYBOARDSTATE *k);
        int msx_cycle(KEYBOARDSTATE *k);
        int konami_cycle(KEYBOARDSTATE *k);
        int title_cycle(KEYBOARDSTATE *k);
        int gamestart_cycle(KEYBOARDSTATE *k);
        int game_cycle(KEYBOARDSTATE *k);
        int gameover_cycle(KEYBOARDSTATE *k);
        int endsequence_cycle(KEYBOARDSTATE *k);
        int titleanimation_cycle(KEYBOARDSTATE *k);
        int interlevel_cycle(KEYBOARDSTATE *k);
        int howtoplay_cycle(KEYBOARDSTATE *k);
        int credits_cycle(KEYBOARDSTATE *k);
        int test_cycle(KEYBOARDSTATE *k);

        void splash_draw(void);
        void msx_draw(void);
        void konami_draw(void);
        void title_draw(void);
        void gamestart_draw(void);
        void game_draw(void);
        void gameover_draw(void);
        void endsequence_draw(void);
        void titleanimation_draw(void);
        void interlevel_draw(void);
        void howtoplay_draw(void);
        void credits_draw(void);
        void test_draw(void);

        int screen_x(int x); /* given a coordinate in 640x480, returns the proper coordinate at the current resolution */
        int screen_y(int y);    /* given a coordinate in 640x480, returns the proper coordinate at the current resolution */

		void draw_square(int x1,int y1,int x2,int y2,float r,float g,float b,float a);

		void getMenu(int menuID);
		void drawMenu(float enterEffect);

		bool keyPress(int key,int player,KEYBOARDSTATE *k);	// if player == -1 then it checks if any of them has pressed the key
		bool keyPressed(int key,int player,KEYBOARDSTATE *k);	// if player == -1 then it checks if any of them has pressed the key

        class GLTManager *m_GLTM;
        class SFXManager *m_SFXM;
        class MusicManager *m_MusicM;

		TTF_Font *m_font16,*m_font32;

		// Store the list of mouse clicks still not processed by the interface:
		List<int> m_mouse_click_x;
		List<int> m_mouse_click_y;

        int m_screen_dx, m_screen_dy;
        int m_state,m_previous_state;
        int m_state_cycle;
        Knightmare *m_game;

		// Input:
        List<class VirtualController> m_vc_l;
		int m_num_joysticks;

        // configuration:
		int m_joystick1_num,m_joystick2_num;
        int m_keys_configuration1[7];
        int m_keys_configuration2[7];
        int m_sfx_volume, m_music_volume;

        // intro screens:
        bool m_skip_intro_screens;

        // Title state:
		int m_mouse_x,m_mouse_y,m_mouse_button;
		int m_draw_mouse_status;	// 0 - not showing, 1 - appearing, 2 - showing , 3 - disappearing
		float m_draw_mouse;
		int m_mouse_movement_timmer;

		List<MenuItem> m_menu;
		int m_title_menu_ID;
		Symbol *m_title_menu_name;
		int m_title_menu_state;
		float m_title_entry_effect;
		int m_title_menu_selected;
		int m_title_menu_action;

		int m_title_menu_redefining_key_player;
		int m_title_menu_redefining_key;

        // Title animation:
		BatEffect *m_bat_effect;
		List<Lightning> m_lightnings;
		float m_title_lighning,m_title1_alpha,m_title2_alpha;

		// Gamestart:
		int m_gamestart_music_channel;

        // Game state:
		int m_n_players;
        int m_game_state;
        int m_current_level;

        // Game over:
        int m_gameover_state;

        // Inter level:
        int m_interlevel_state;
        int m_interlevel_cycle;

        // How to play:
        int m_howtoplay_state;
        int m_howtoplay_cycle;
        int m_howtoplay_difficulty;
        Knightmare *m_test_game;

		// credits:
        int m_credits_state;
        int m_credits_cycle;

        // end sequence:
        int m_endsequence_state;
        int m_endsequence_cycle;

		// Highscores:
		Symbol **m_highscore_names_1player[MAX_HIGHSCORES];
		Symbol **m_highscore_names_2players[MAX_HIGHSCORES];
		int m_highscore_scores_1player[MAX_HIGHSCORES];
		int m_highscore_scores_2players[MAX_HIGHSCORES];

        // Hiscores
        int get_hiscore(void)
        {
            return 0;
        };
        void set_hiscore(int newscore, Symbol *player_name)
        {
			// ...
        };
};

#endif

